#!/usr/local/bin/python
# encoding: utf-8
"""
*Generate various sky-projection maps given a Healpix likeihood map*

:Author:
    David Young

:Date Created:
    June 30, 2016
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools
from plot_wave_observational_timelines import plot_wave_observational_timelines


class projections():
    """
    *The worker class for the projections module*

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``gwid`` -- the map's gravitational wave ID 
        - ``healpixPath`` -- path to the LV Healpix likelihood map.
        - ``projection`` -- the desired projection to warp the map to. Default *mollweide* (nothing else added yet)
        - ``outputDirectory`` -- path to the directory to output the map to. 

    **Usage:**
        .. todo::

            - add usage info
            - create a sublime snippet for usage

        .. code-block:: python 

            usage code   

    .. todo::

        - @review: when complete, clean projections class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    def __init__(
            self,
            log,
            gwid,
            healpixPath,
            outputDirectory,
            settings=False,
            projection="mollweide",
    ):
        self.log = log
        log.debug("instansiating a new 'projections' object")
        self.settings = settings
        self.healpixPath = healpixPath
        self.projection = projection
        self.outputDirectory = outputDirectory
        self.gwid = gwid

        # xt-self-arg-tmpx

        # 2. @flagged: what are the default attrributes each object could have? Add them to variable attribute set here
        # Variable Data Atrributes

        # 3. @flagged: what variable attrributes need overriden in any baseclass(es) used
        # Override Variable Data Atrributes

        # Initial Actions
        plotter = plot_wave_observational_timelines(
            log=log,
            settings=settings
        )

        plotter.generate_probability_plot(
            gwid=gwid,
            pathToProbMap=healpixPath,
            fileFormats=["pdf"],
            outputDirectory=outputDirectory,
            projection=projection,
            plotType="timeline"
        )

        return None

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the projections object*

        **Return:**
            - ``projections``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        projections = None

        self.log.info('completed the ``get`` method')
        return projections

    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx



import os
import nose
import shutil
import yaml
from breaker.utKit import utKit

from fundamentals import tools

su = tools(
    arguments={"settingsFile": None},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName="breaker"
)
arguments, settings, log, dbConn = su.setup()

# load settings
stream = file(
    "/Users/Dave/.config/breaker/breaker.yaml", 'r')
settings = yaml.load(stream)
stream.close()

# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()


class test_projections():

    def test_projections_function(self):

        from breaker.plots import projections
        this = projections(
            log=log,
            gwid="GW151226",
            healpixPath=pathToInputDir + "/GW151226-LALInference_skymap_final.fits",
            projection="mollweide",
            outputDirectory=pathToOutputDir
        )
        this.get()

    # def test_projections_function2(self):

    #     from breaker.plots import projections
    #     this = projections(
    #         log=log,
    #         gwid="GW151226",
    #         healpixPath=pathToInputDir + "/GW151226-LALInference_skymap_final.fits",
    #         projection="aitoff",
    #         outputDirectory=pathToOutputDir
    #     )
    #     this.get()

    # def test_projections_function3(self):

    #     from breaker.plots import projections
    #     this = projections(
    #         log=log,
    #         gwid="GW151226",
    #         healpixPath=pathToInputDir + "/GW151226-LALInference_skymap_final.fits",
    #         projection="hammer",
    #         outputDirectory=pathToOutputDir
    #     )
    #     this.get()

    # def test_projections_function4(self):

    #     from breaker.plots import projections
    #     this = projections(
    #         log=log,
    #         gwid="GW151226",
    #         healpixPath=pathToInputDir + "/GW151226-LALInference_skymap_final.fits",
    #         projection="lambert",
    #         outputDirectory=pathToOutputDir
    #     )
    #     this.get()

    # def test_projections_function5(self):

    #     from breaker.plots import projections
    #     this = projections(
    #         log=log,
    #         gwid="GW151226",
    #         healpixPath=pathToInputDir + "/GW151226-LALInference_skymap_final.fits",
    #         projection="polar",
    #         outputDirectory=pathToOutputDir
    #     )
    #     this.get()

    # def test_projections_function6(self):

    #     from breaker.plots import projections
    #     this = projections(
    #         log=log,
    #         gwid="GW151226",
    #         healpixPath=pathToInputDir + "/GW151226-LALInference_skymap_final.fits",
    #         projection="rectilinear",
    #         outputDirectory=pathToOutputDir
    #     )
    #     this.get()

    def test_projections_function_exception(self):

        from breaker.plots import projections
        try:
            this = projections(
                log=log,
                settings=settings,
                fakeKey="break the code"
            )
            this.get()
            assert False
        except Exception, e:
            assert True
            print str(e)

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function
